package ovh.herisson.hornofthewild;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemUsage;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.registry.tag.TagKey;
import net.minecraft.sound.SoundCategory;
import net.minecraft.sound.SoundEvents;
import net.minecraft.text.Text;
import net.minecraft.util.Hand;
import net.minecraft.util.Identifier;
import net.minecraft.util.TypedActionResult;
import net.minecraft.util.UseAction;
import net.minecraft.util.math.BlockBox;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class HornOfTheWildItem extends Item {
    public static final TagKey<Block> HORN_BREAKABLE = TagKey.of(RegistryKeys.BLOCK, new Identifier(HornOfTheWild.MOD_ID, "hotw_breakable"));
    public HornOfTheWildItem(Settings settings) {
        super(settings);
    }

    @Override
    public int getMaxUseTime(ItemStack itemStack) {
        return 72000;
    }

    @Override
    public TypedActionResult<ItemStack> use(World world, PlayerEntity playerEntity, Hand hand) {
        return ItemUsage.consumeHeldItem(world, playerEntity, hand);
    }

    @Override
    public void usageTick(World world, LivingEntity livingEntity, ItemStack itemStack, int time) {
        if (!world.isClient) {
            if(time % 5 == 0){
               HornBeat(world, itemStack, livingEntity.getBlockPos(), livingEntity);
            }
            world.playSound(null, livingEntity.getX(), livingEntity.getY(), livingEntity.getZ(), SoundEvents.ENTITY_HORSE_BREATHE, SoundCategory.PLAYERS, 1.0F, 1.0F);
        }
    }

    @Override
    public UseAction getUseAction(ItemStack itemStack) {
        return UseAction.BOW;
    }

    private static boolean canHarvest(World world, ItemStack stack, BlockPos pos, LivingEntity livingEntity) {
        BlockState state = world.getBlockState(pos);
        return state.isIn(HORN_BREAKABLE);
    }

    private static void HornBeat(World world, ItemStack stack, BlockPos srcPos, LivingEntity livingEntity){
        int range = 12;
        int rangeY = 3;

        List<BlockPos> coords = new ArrayList<>();
        for (BlockPos pos: BlockPos.iterate(srcPos.add(-range, -rangeY, -range), srcPos.add(range, rangeY, range))){
            if(HornOfTheWildItem.canHarvest(world, stack, pos, livingEntity)){
                coords.add(pos.toImmutable());
            }
        }

        Collections.shuffle(coords);

        int count = Math.min(coords.size(), 32);
        for (int i = 0; i < count; i++) {
            BlockPos currCoords = coords.get(i);
            BlockState state = world.getBlockState(currCoords);
            BlockEntity be = world.getBlockEntity(currCoords);
            world.breakBlock(currCoords, true);
        }

    }
}
